# -*- coding: utf-8 -*-


class FrontendKeys(object):
    # string, args: user_type; user.id_ expiration: user token's expiration, msgpack encoded
    user = 'user:{}:{}'


class WechatAppKeys(object):
    # string
    access_token = 'access_token'



# -*- coding: utf-8 -*-

import base64
import json
import logging
import traceback

from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.padding import PKCS7

from ...exceptions import BaseAppException, UtilsException
from .base import BaseAdaptor

logger = logging.getLogger(__name__)


class UserAdaptor(BaseAdaptor):
    name = 'user'

    jscode2session_url = 'https://api.weixin.qq.com/sns/jscode2session'

    def code_to_session_info(self, code):
        try:
            params = {
                'appid': self.wxapp_id,
                'secret': self.wxapp_secret,
                'grant_type': 'authorization_code',
                'js_code': code,
            }
            ret = self.http_client.get(self.jscode2session_url, params=params)
            errcode = ret.get('errcode')
            if errcode:
                logger.error(
                    'Tencent return error while exchanging code for session_info. code: %s msg: %s',
                    errcode,
                    ret.get('errmsg'),
                )
                raise UtilsException(
                    return_data={
                        'errcode': errcode,
                        'errmsg': ret.get('errmsg'),
                    },
                    errcode=992101,
                )
        except BaseAppException:
            raise
        except:
            logger.error(
                'Error while exchanging code for session_info: %s Err: %s',
                code,
                traceback.format_exc(),
            )
            raise UtilsException(
                errcode=992150,
            )
        return ret

    def decrypt(self, session_key, iv, encrypted):
        try:
            key = base64.b64decode(session_key)
            iv = base64.b64decode(iv)
            cipher = Cipher(
                algorithms.AES(key),
                modes.CBC(iv),
                backend=default_backend(),
            )
            decryptor = cipher.decryptor()
            plain = decryptor.update(base64.b64decode(encrypted)) + decryptor.finalize()
            unpadder = PKCS7(128).unpadder()
            decrypted = unpadder.update(plain)
            decrypted += unpadder.finalize()
            decrypted = json.loads(decrypted.decode('utf8'))
        except UtilsException:
            raise
        except:
            logger.error(
                'Error while decrypting message(simple): %s Err: %s',
                encrypted,
                traceback.format_exc(),
            )
            raise UtilsException(
                errcode=992250,
            )
        return decrypted



# -*- coding: utf-8 -*-

from .base import BaseAppException


class UtilsException(BaseAppException):
    # category
    # utils: 99
    category = 'utils'

    # sub category:
    #   - http.get 10
    #   - http.post 11
    #   - wechat.access_token.retrieving 20
    #   - wechat.user.exchange_code_for_session_info 21
    #   - wechat.user.decryption 22
    #   - wechat.qrcode.generation 23

    co_msg_mapping = {
        991050: {
            'message': 'Unexpected exception',
            'http_code': 500,
            'sub_category': 'http.get'
        },
        991150: {
            'message': 'Unexpected exception',
            'http_code': 500,
            'sub_category': 'http.post'
        },
        992001: {
            'message': 'Tencent returned error',
            'http_code': 500,
            'sub_category': 'wechat.access_token.retrieving'
        },
        992002: {
            'message': 'Missing access token in tencent returned data',
            'http_code': 400,
            'sub_category': 'wechat.access_token.retrieving'
        },
        992050: {
            'message': 'Unexpected exception',
            'http_code': 500,
            'sub_category': 'wechat.access_token.retrieving'
        },
        992051: {
            # loading access token from redis
            'message': 'Unexpected exception',
            'http_code': 500,
            'sub_category': 'wechat.access_token.retrieving'
        },
        992052: {
            # persisting access token from redis
            'message': 'Unexpected exception',
            'http_code': 500,
            'sub_category': 'wechat.access_token.retrieving'
        },
        992101: {
            'message': 'Wechat returns error',
            'http_code': 500,
            'sub_category': 'wechat.user.exchange_code_for_session_info',
        },
        992150: {
            'message': 'Unexpected exception',
            'http_code': 500,
            'sub_category': 'wechat.user.exchange_code_for_session_info',
        },
        992250: {
            'message': 'Unexpected exception',
            'http_code': 500,
            'sub_category': 'wechat.user.decryption',
        },
        992301: {
            'message': 'Illegal char in scene',
            'http_code': 400,
            'sub_category': 'wechat.qrcode.generation',
        },
        992302: {
            'message': 'Retrieving access token failed',
            'http_code': 400,
            'sub_category': 'wechat.qrcode.generation',
        },
        992303: {
            'message': 'Wechat returns error',
            'http_code': 400,
            'sub_category': 'wechat.qrcode.generation',
        },
        992350: {
            'message': 'Unexpected exception',
            'http_code': 500,
            'sub_category': 'wechat.qrcode.generation',
        },
    }



# -*- coding: utf-8 -*-

from werkzeug.exceptions import HTTPException


class BaseAppException(HTTPException):
    category = None
    co_msg_mapping = {}

    def __init__(self, errcode, message=None, sub_category=None, params=None,
                 http_code=None, description=None, return_data=None):
        co_info = self.co_msg_mapping.get(errcode, {})
        if message is None:
            message = co_info.get('message')
        if description is None:
            description = co_info.get('description')
        if http_code is None:
            http_code = co_info.get('http_code')
        if sub_category is None:
            sub_category = co_info.get('sub_category')

        super().__init__(description=description)

        self.code = http_code
        self.errcode = errcode
        self.message = message

        self.sub_category = sub_category
        self.params = params
        self.data = {
            'message': self.message,
            'errcode': self.errcode,
            'category': self.category,
            'sub_category': self.sub_category,
        }

        if return_data is None:
            return_data = {}
        self.return_data = return_data
        if return_data is not None and isinstance(return_data, dict):
            self.data.update(data=return_data)

        if params is not None and isinstance(params, dict):
            self.data.update(params=params)

    def __repr__(self):
        return '{}: category: {}; sub_category: {}; errcode: {}; message: {}; data: {}'.format(
            self.__class__.__name__,
            self.category,
            self.sub_category,
            self.errcode,
            self.message,
            self.data,
        )



# -*- coding: utf-8 -*-

from flask import current_app
from flask_restplus import Resource, reqparse

from .adaptor import current_user, admin_required, login_required


class BaseResource(Resource):
    def __init__(self, *args, **kwargs):
        from application import db
        super().__init__(*args, **kwargs)
        self.parser = reqparse.RequestParser()
        self.app = current_app
        self.user = current_user
        self.db = db


class AdminResource(BaseResource):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.method_decorators = [admin_required]


class LoggedInUserResource(BaseResource):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.method_decorators = [login_required]



# -*- coding: utf-8 -*-

import os
import copy
import logging
from logging import config

from . import config_data

logger_sub_dirs = [
    'utils',
    'auth',
]

class AppLogger(object):
    logger_sub_dirs = []

    def __init__(self, app=None):
        self.app = None
        self._logger_dir = None

        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app

        self.init_dirs()

        self._set_root_logger()

    def init_dirs(self):
        if not self.logger_dir:
            return

        for sub_dir in (*self.logger_sub_dirs, *logger_sub_dirs):
            sub_dir = os.path.join(self.logger_dir, sub_dir)
            if not os.path.exists(sub_dir):
                os.makedirs(sub_dir)

    @property
    def logger_dir(self):
        return self._logger_dir

    @property
    def config(self):
        return {}

    def _set_root_logger(self):
        config = self._get_config()
        configuration = self._merge_dict_config(config, self.config)
        logging.config.dictConfig(configuration)

    def _get_config(self):
        if self.logger_dir:
            handler_class = 'logging.handlers.TimedRotatingFileHandler'
            backup_count = 10
            when = 'midnight'
        else:
            handler_class = 'logging.StreamHandler'
            stream = 'ext://sys.stdout'

        handlers = copy.copy(config_data.handlers)
        logging_config = copy.copy(config_data.logging_config)

        if self.logger_dir:
            backup_count = 10
            when = 'midnight'

            handlers['utils_debug']['class'] = handler_class
            handlers['utils_debug']['filename'] = os.path.join(self.logger_dir, 'utils', 'debug.log')
            handlers['utils_debug']['backupCount'] = backup_count
            handlers['utils_debug']['when'] = when

            handlers['utils_warning']['class'] = handler_class
            handlers['utils_warning']['filename'] = os.path.join(self.logger_dir, 'utils', 'warning.log')
            handlers['utils_warning']['backupCount'] = backup_count
            handlers['utils_warning']['when'] = when

            handlers['auth_debug']['class'] = handler_class
            handlers['auth_debug']['filename'] = os.path.join(self.logger_dir, 'auth', 'debug.log')
            handlers['auth_debug']['backupCount'] = backup_count
            handlers['auth_debug']['when'] = when

            handlers['auth_warning']['class'] = handler_class
            handlers['auth_warning']['filename'] = os.path.join(self.logger_dir, 'auth', 'warning.log')
            handlers['auth_warning']['backupCount'] = backup_count
            handlers['auth_warning']['when'] = when
        else:
            stream = 'ext://sys.stdout'

            handlers['utils_debug']['stream'] = stream
            handlers['utils_warning']['stream'] = stream
            handlers['auth_debug']['stream'] = stream
            handlers['auth_warning']['stream'] = stream

        logging_config['handlers'] = handlers

        return logging_config

    def _merge_dict_config(self, overridden, overriding):
        if overriding is None or not isinstance(overriding, dict):
            overriding = {}

        formatters = copy.copy(overridden.get('formatters', {}))
        formatters.update(overriding.get('formatters', {}))

        handlers = copy.copy(overridden.get('handlers', {}))
        handlers.update(overriding.get('handlers', {}))

        loggers = copy.copy(overridden.get('loggers', {}))
        loggers.update(overriding.get('loggers', {}))

        configuration = {
            'version': 1,
            'disable_existing_loggers': False,
            'formatters': formatters,
            'handlers': handlers,
            'loggers': loggers,
        }

        return configuration



# -*- coding: utf-8 -*-

from functools import wraps
import logging
import traceback

from flask import has_request_context, _request_ctx_stack, current_app, request
from werkzeug.local import LocalProxy
from werkzeug.utils import import_string
from itsdangerous import TimedJSONWebSignatureSerializer, SignatureExpired, BadSignature, BadTimeSignature

from flask_restplus import reqparse, Resource

from ...exceptions import AuthException

logger = logging.getLogger(__name__)

current_user = LocalProxy(lambda: _get_user())


class Auth(object):
    def __init__(self, app=None):
        self.app = None

        self.secret_key = None
        self.session_expiration = None
        self.serializer = None

        self.user_cls = None
        self.admin_cls = None

        self.parser = reqparse.RequestParser()
        self.parser.add_argument(
            'Authorization',
            location='headers',
            dest='token',
            required=True,
            help='Missing session token data',
        )

        self.wechat = None

        self.redis = None

        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.secret_key = app.config['SECRET_KEY']
        self.session_expiration = app.config['SESSION_EXPIRATION']
        logger.info('Set serializer expiration: %s', self.session_expiration)

        self.serializer = TimedJSONWebSignatureSerializer(
            self.secret_key,
            self.session_expiration,
        )

        self.user_cls = import_string(
            app.config.setdefault('AUTH_USER_CLASS', 'models.User')
        )
        self.admin_cls = import_string(
            app.config.setdefault('AUTH_ADMIN_CLASS', 'models.Admin')
        )

        self.wechat = app.wechat
        self.redis = app.redis

        self.app = app
        app.auth = self

    def _get_wx_session_info(self, code):
        try:
            session_info = self.wechat.code_to_session_info(code=code)
        except:
            logger.error(
                'Exchange session key FAILED. %s Err: %s',
                code,
                traceback.format_exc(),
            )
            raise AuthException(
                errcode=981201,
            )
        else:
            logger.debug('Retrieved user info %s -> %s', code, session_info)
        return session_info

    def login(self, code=None, username=None, password=None):
        if not code and not username and not password:
            logger.warning('Missing login credentials, EXIT')
            raise AuthException(
                errcode=981101,
            )
        if code is not None:
            session_info = self._get_wx_session_info(code)
            user = self.user_cls.login(session_info)
            try:
                user_type = user.user_type or 'wechat'
            except:
                user_type = 'wechat'
            token = self.serializer.dumps({
                'uid': user.id_,
                'user_type': user_type,
                'openid': user.openid,
                'site': 'annualReport'
            }).decode('utf8')
        else:
            user = self.admin_cls.login(username, password)
            try:
                user_type = user.user_type or 'admin'
            except:
                user_type = 'admin'
            token = self.serializer.dumps({
                'uid': user.id_,
                'user_type': user_type,
                'site': 'annualReport',
            }).decode('utf8')
        logger.debug(
            'Generated token. code: %s username: %s token: %s',
            code,
            username,
            token,
        )
        self.redis.login_user(user, user_type)
        return user, token

    def verify_session_token(self, token):
        try:
            data = self.serializer.loads(token)
        except SignatureExpired:
            logger.info(
                'Token expired for %s',
                token,
            )
            raise AuthException(
                errcode=981102,
            )
        except (BadSignature, BadTimeSignature):
            logger.info(
                'Token data corrupted: %s',
                token,
            )
            raise AuthException(
                errcode=981103,
            )
        logger.debug('Deserialized user token: %s -> %s', token, data)
        return self._get_user_from_token(data)

    def _get_user_from_token(self, data):
        user_type = data.get('user_type')
        uid = data.get('openid') or data.get('uid')
        site = data.get('site')

        if not uid:
            raise AuthException(
                errcode=981104,
            )
        user_info = self.redis.load_user(uid, user_type)

        if user_type in ('wechat',):
            if user_info is not None and isinstance(user_info, dict):
                user = self.user_cls.get_proxy(**user_info)
                if user is not None:
                    user.login_type = 'wechat'
                    logger.debug('Loaded wechat user proxy for %s', uid)
                    return user
                logger.error(
                    'Getting wechat user proxy failed, trying loading from db %s',
                    uid,
                )

            user = self.user_cls.query.filter(self.user_cls.openid == uid).first()
            if user is None:
                raise AuthException(
                    errcode=981203,
                    params={
                        'uid': uid,
                        'user_type': user_type,
                    }
                )
            else:
                user.login_type = 'wechat'
                self.redis.login_user(user, user_type)
        elif user_type == 'admin':
            if user_info is not None and isinstance(user_info, dict):
                admin = self.admin_cls.get_proxy(**user_info)
                if admin is not None:
                    admin.login_type = 'admin'
                    logger.debug('Loaded admin proxy for %s', uid)
                    return admin
                logger.error(
                    'Getting admin proxy failed, trying loading from db %s',
                    uid,
                )

            user = self.admin_cls.query.filter(self.admin_cls.id_ == uid).first()
            if user is None:
                raise AuthException(
                    errcode=981305,
                    params={
                        'uid': uid,
                        'user_type': user_type,
                    },
                )
            else:
                user.login_type = 'admin'
                self.redis.login_user(user, user_type)
        else:
            raise AuthException(
                errcode=981105,
                params={
                    'user_type': user_type,
                }
            )

        return user


def _get_user():
    if has_request_context() and hasattr(_request_ctx_stack.top, 'user'):
        return getattr(_request_ctx_stack.top, 'user')

    try:
        try:
            args = current_app.auth.parser.parse_args()
        except:
            logger.error(
                'Parsing headers error, headers(auth): %s Err: %s',
                request.headers.get('authorization'),
                traceback.format_exc(),
            )
            raise AuthException(
                errcode=981106,
            )
        token = args.get('token')
        try:
            _, token = token.split(' ')
        except:
            logger.error(
                'Token format error %s Err: 5s',
                token,
                traceback.format_exc(),
            )
            raise AuthException(
                errcode=981107,
            )
        else:
            logger.debug('Retrieved user token from header: %s', token)

        user = current_app.auth.verify_session_token(token)
    except AuthException:
        raise

    setattr(_request_ctx_stack.top, 'user', user)
    return user


def admin_required(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        if not current_user.is_admin:
            raise AuthException(
                errcode=981401,
            )
        return func(*args, **kwargs)
    return wrapper


def login_required(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        current_user._get_current_object()
        return func(*args, **kwargs)
    return wrapper


class BaseResource(Resource):
    def __init__(self, *args, **kwargs):
        from application import db
        super().__init__(*args, **kwargs)
        self.parser = reqparse.RequestParser()
        self.app = current_app
        self.user = current_user
        self.db = db


class AdminResource(BaseResource):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.method_decorators = [admin_required]


class LoggedInUserResource(BaseResource):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.method_decorators = [login_required]



# -*- coding: utf-8 -*-

import os
import time


BASE_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

sentinel = object


def now_ts():
    return int(time.time())


def nop():
    return



# -*- coding: utf-8 -*-

import logging
import traceback

import requests

from ..exceptions import UtilsException

logger = logging.getLogger(__name__)


class BaseHttpClient(object):
    def __init__(self):
        self._session = None

    @property
    def session(self):
        if self._session is None:
            self._session = requests.Session()
        return self._session

    def get(self, url, params=None, jsonify=True, binary=False):
        try:
            resp = self.session.get(url, params=params)
            resp.raise_for_status()
            if jsonify:
                resp = resp.json()
            elif binary:
                resp = resp.content
            else:
                resp = resp.text
        except:
            logger.error(
                'Error while request(get) url: %s params: %s Err: %s',
                url,
                params,
                traceback.format_exc(),
            )
            raise UtilsException(
                errcode=983050,
                params={
                    'url': url,
                    'params': params,
                }
            )
        return resp

    def post(self, url, data=None, json_payload=None, params=None, headers=None, jsonify=True, raw=False):
        try:
            if headers is None:
                headers = {}
            headers.update(self.session.headers)
            resp = self.session.post(url, data=data, json=json_payload, headers=headers, params=params)
            resp.raise_for_status()
            if raw:
                pass
            elif jsonify:
                resp = resp.json()
            else:
                resp = resp.text
        except:
            logger.error(
                'Error while request(post) url: %s data: %s json: %s params: %s Err: %s',
                url,
                data,
                json_payload,
                params,
                traceback.format_exc()
            )
            raise UtilsException(
                errcode=991150,
                params={
                    'url': url,
                    'data': data,
                    'json': json_payload,
                    'params': params,
                }
            )
        return resp



# -*- coding: utf-8 -*-

import logging
import traceback

from ...exceptions import BaseAppException, UtilsException
from .base import BaseAdaptor

logger = logging.getLogger(__name__)


class QrcodeAdaptor(BaseAdaptor):
    name = 'qrcode'
    qrcode_url = 'https://api.weixin.qq.com/wxa/getwxacodeunlimit'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.qrcode_allowed_char = "!#$&'()*+,/:;=?@-._~" \
                                   + ''.join(chr(ord('a') + i) for i in range(26)) \
                                   + ''.join(chr(ord('A') + i) for i in range(26)) \
                                   + ''.join(str(i) for i in range(10))

    @property
    def access_token(self):
        return self.wrapper.access_token

    def get_wechat_qrcode(self, scene_data, page=None, data=None):
        try:
            if isinstance(scene_data, dict):
                scene_data = ['='.join((k, str(v))) for k, v in scene_data.items()]
                scene = ','.join(scene_data)
            else:
                scene = str(scene_data)
            for i in scene:
                if i not in self.qrcode_allowed_char:
                    logger.error(
                        'Illegal char. EXIT. %s',
                        scene,
                    )
                    raise UtilsException(
                        params={
                            'scene': scene,
                        },
                        errcode=992301,
                    )

            try:
                access_token = self.access_token
            except BaseAppException:
                logger.error(
                    'Retrieving access token failed. EXIT.'
                )
                raise UtilsException(
                    errcode=992302,
                )
            params = {
                'access_token': access_token,
            }
            data = data or {}
            data.update(scene=scene)
            if page:
                data.update(page=page)

            ret = self.http_client.post(
                self.qrcode_url,
                json_payload=data,
                params=params,
                raw=True
            )
            try:
                error = ret.json()
            except:
                pass
            else:
                logger.error(
                    'Wechat returned error while retrieving share-qrcode. params: %s err_info: %s',
                    params,
                    error,
                )
                raise UtilsException(
                    message=error.get('errmsg'),
                    errcode=992303,
                )
        except BaseAppException:
            raise
        except:
            logger.error(
                'Error while retrieving qrcode from wechat. %s %s %s Err: %s',
                scene_data,
                page,
                data,
                traceback.format_exc(),
            )
            raise UtilsException(
                errcode=992350,
            )
        else:
            return ret.content



# -*- coding: utf-8 -*-


from ...adaptors import AccessTokenAdaptor


class FlaskAccessTokenAdaptor(AccessTokenAdaptor):
    def register(self, **kwargs):
        self.access_token_default_expiration = self.app.config['TENCENT_ACCESS_TOKEN_DEFAULT_EXPIRATION']
        self.access_token_expiration_tolerance = self.app.config['TENCENT_ACCESS_TOKEN_EXPIRATION_TOLERANCE']



# -*- coding: utf-8 -*-

from .base import BaseHttpClient


class FlaskHttpClient(BaseHttpClient):
    def __init__(self, app=None):
        super().__init__()
        self.app = None

        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app
        app.http_client = self



# -*- coding: utf-8 -*-

import logging
import traceback

from ...exceptions import BaseAppException, UtilsException
from .base import BaseAdaptor

logger = logging.getLogger(__name__)


class AccessTokenAdaptor(BaseAdaptor):
    name = 'access_token'
    access_token_url = 'https://api.weixin.qq.com/cgi-bin/token'

    def __init__(self, wrapper):
        self.access_token_default_expiration = 60 * 60 * 2
        self.access_token_expiration_tolerance = 200

        super().__init__(wrapper)

    @property
    def access_token(self):
        try:
            token = self.redis.load_access_token()
            if token:
                return token
            logger.info('Token may expired, retrieving it from tencent')
            token, expiration = self._get_access_token()
            try:
                self.redis.persist_access_token(token, expiration)
            except:
                pass
        except BaseAppException:
            raise
        except:
            logger.error(
                'Error while retrieving access token: %s',
                traceback.format_exc(),
            )
            raise UtilsException(
                errcode=992050,
            )
        else:
            return token

    def _get_access_token(self):
        params = {
            'grant_type': 'client_credential',
            'appid': self.wxapp_id,
            'secret': self.wxapp_secret,
        }
        ret = self.http_client.get(self.access_token_url, params=params)
        errcode = ret.get('errcode')
        if errcode:
            logger.error(
                'Tencent return error while retrieving access token. code: %s msg: %s',
                errcode,
                ret.get('errmsg'),
            )
            raise UtilsException(
                errcode=992001,
                return_data={
                    'errcode': errcode,
                    'errmsg': ret.get('errmsg'),
                }
            )
        access_token = ret.get('access_token')
        if not access_token:
            raise UtilsException(
                errcode=992002,
            )

        expiration = ret.get('expires_in', self.access_token_default_expiration)
        try:
            expiration = int(expiration)
        except:
            expiration = self.access_token_default_expiration
        expiration -= self.access_token_expiration_tolerance

        return access_token, expiration



# -*- coding: utf-8 -*-

handlers = {
    'utils_debug': {
        'level': 'DEBUG',
        'filters': None,
        'formatter': 'webutils',
    },
    'utils_warning': {
        'level': 'WARNING',
        'filters': None,
        'formatter': 'webutils',
    },
    'auth_debug': {
        'level': 'DEBUG',
        'filters': None,
        'formatter': 'webutils',
    },
    'auth_warning': {
        'level': 'WARNING',
        'filters': None,
        'formatter': 'webutils',
    },
}

logging_config = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'webutils': {
            'format': '[%(levelname).1s]%(asctime)s [%(name)s]: %(message)s',
        },
    },

    'loggers': {
        'utils': {
            'handlers': ['utils_debug', 'utils_warning'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'utils.auth': {
            'handlers': ['auth_debug', 'auth_warning'],
            'level': 'DEBUG',
            'propagate': True,
        },
    },
}



# -*- coding: utf-8 -*-


class BaseAdaptor(object):
    name = None

    def __init__(self, wrapper, **kwargs):
        self.wrapper = wrapper
        self.register(**kwargs)

    @property
    def app(self):
        return self.wrapper.app

    @property
    def wxapp_id(self):
        return self.wrapper.wxapp_id

    @property
    def wxapp_secret(self):
        return self.wrapper.wxapp_secret

    @property
    def redis(self):
        return self.wrapper.redis

    @property
    def http_client(self):
        return self.wrapper.http_client

    def register(self, **kwargs):
        # initializing config with wrapper here
        pass



# -*- coding: utf-8 -*-

__all__ = [
    'Auth',
    'current_user',
    'Resource',
    'AdminResource',
    'LoggedInUserResource',
]

from .adaptor import Auth, current_user
from .resources import BaseResource as Resource, AdminResource, LoggedInUserResource



# -*- coding: utf-8 -*-

from ..adaptors import UserAdaptor, QrcodeAdaptor
from .adaptors import FlaskAccessTokenAdaptor


class Wechat(object):
    registering_adaptors = (
        FlaskAccessTokenAdaptor,
        UserAdaptor,
        QrcodeAdaptor,
    )

    def __init__(self, app=None):
        self.app = None
        self.redis = None
        self.http_client = None

        self.wxapp_id = None
        self.wxapp_secret = None

        self.adaptors = {}

        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.redis = app.redis
        self.http_client = app.http_client

        self.wxapp_id = app.config['WXAPP_ID']
        self.wxapp_secret = app.config['WXAPP_SECRET']

        self.app = app
        app.wechat = self

        self.register_adaptor()

    def register_adaptor(self):
        for adaptor in self.registering_adaptors:
            self.adaptors[adaptor.name] = adaptor(self)

    @property
    def access_token(self):
        return self.adaptors['access_token'].access_token

    def decrypt_user_info(self, session_key, iv, encrypted):
        return self.adaptors['user'].decrypt(session_key, iv, encrypted)

    def code_to_session_info(self, code):
        return self.adaptors['user'].code_to_session_info(code)

    def get_wechat_qrcode(self, scene_data, page=None, data=None):
        return self.adaptors['qrcode'].get_wechat_qrcode(self, scene_data, page, data)



import setuptools


setuptools.setup(
    name="webutils-wechat",
    version="0.0.2",
    author="SimonQ",
    author_email="simonq.mq@gmail.com",
    description="Simple utils for Restful-web-wechat",
    long_description="Simple utils for Restful-web-wechat",
    long_description_content_type="text/markdown",
    url="https://gitee.com/xxmeng/webutils-wechat",
    packages=setuptools.find_packages(),
    install_requires=[
        'flask==1.0.2',
        'flask_restplus==0.12.1',
        'requests==2.21.0',
        'redis==2.10.6',
        'msgpack==0.5.6',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding: utf-8 -*-

import logging
import traceback

import msgpack

from ..exceptions import UtilsException
from .constants import FrontendKeys, WechatAppKeys


logger = logging.getLogger(__name__)


class RedisClient(object):
    def __init__(self):
        self.client = None

        self.host = None
        self.port = None
        self.db = None
        self.password = None
        self.pool = None

        self.user_expiration = None
        self.user_id_attr_name = None
        self.admin_id_attr_name = None

    def _get_user_id(self, user):
        if self.user_id_attr_name:
            return getattr(user, self.user_id_attr_name, None)
        return getattr(user, 'id_', None) or getattr(user, 'id', None) or getattr(user, 'pk', None)

    def _get_admin_id(self, user):
        if self.admin_id_attr_name:
            return getattr(user, self.admin_id_attr_name, None)
        return getattr(user, 'id_', None) or getattr(user, 'id', None) or getattr(user, 'pk', None)

    def login_user(self, user, user_type=None):
        if user_type is None:
            user_type = user.user_type
        if user_type == 'wechat':
            uid = self._get_user_id(user)
        else:
            uid = self._get_admin_id(user)
        try:
            key = FrontendKeys.user.format(user_type, uid)
            payload = msgpack.dumps(user.redis_payload)
            self.client.setex(key, self.user_expiration, payload)
        except:
            logger.error(
                'Persisting user info to redis error, user: %s Err: %s',
                uid,
                traceback.format_exc(),
            )
            return False
        else:
            logger.debug(
                'Persisted user info to redis. user: %s',
                uid,
            )
            return True

    def load_user(self, uid, user_type):
        try:
            key = FrontendKeys.user.format(user_type, uid)
            info = self.client.get(key)
            if info is not None:
                info = msgpack.loads(info, raw=False)
        except:
            logger.error(
                'Loading user info from redis error. user: %s user_type: %s Err: %s',
                uid,
                user_type,
                traceback.format_exc(),
            )
            return None
        else:
            logger.debug(
                "Loaded user info from redis. user: %s %s -> %s",
                uid,
                user_type,
                info,
            )
            return info

    def persist_access_token(self, access_token, expiration):
        try:
            key = WechatAppKeys.access_token
            self.client.setex(key, expiration, access_token)
        except:
            logger.error(
                'Error while persist wechat access token. access_token: %s Err: %s',
                access_token,
                traceback.format_exc(),
            )
            raise UtilsException(
                errcode=992052,
            )

    def load_access_token(self):
        try:
            key = WechatAppKeys.access_token
            access_token = self.client.get(key)
            if access_token is not None:
                access_token = access_token.decode('utf8')
        except:
            logger.error(
                'Error while loading access token from redis: %s',
                traceback.format_exc(),
            )
            raise UtilsException(
                errcode=992051,
            )
        else:
            return access_token



# -*- coding: utf-8 -*-

import redis

from ..client import RedisClient


class FlaskRedisClient(RedisClient):
    def __init__(self, app=None):
        super().__init__()

        self.app = None

        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.host = app.config.setdefault('REDIS_HOST', 'localhost')
        self.port = app.config.setdefault('REDIS_PORT', 6379)
        self.db = app.config.setdefault('REDIS_DB', 7)
        self.password = app.config.setdefault('REDIS_PASSWORD', None)

        self.pool = redis.ConnectionPool(
            host=self.host,
            port=self.port,
            db=self.db,
            password=self.password,
        )
        self.client = redis.StrictRedis(connection_pool=self.pool)

        self.user_expiration = app.config.setdefault('USER_REDIS_EXPIRATION', 60 * 60 * 24 * 2)
        self.user_id_attr_name = app.config.get('USER_ID_ATTR_NAME')
        self.admin_id_attr_name = app.config.get('ADMIN_ID_ATTR_NAME')

        self.app = app
        app.redis = self
        app.redis_cli = self.client



# -*- coding: utf-8 -*-

from .base import BaseAppException


class AuthException(BaseAppException):
    # category
    # auth: 98
    category = 'web-auth'

    # sub category:
    #   - admin.creation 10
    #   - login 11
    #   - login.wechat 12
    #   - login.admin 13
    #   - authorization 14
    co_msg_mapping = {
        981001: {
            'message': 'username existed',
            'http_code': 400,
            'sub_category': 'admin.creation'
        },
        981002: {
            'message': 'role not exists',
            'http_code': 400,
            'sub_category': 'admin.creation'
        },
        981051: {
            'message': 'db error while creating admin',
            'http_code': 500,
            'sub_category': 'admin.creation'
        },

        981101: {
            'message': 'Missing login credentials',
            'http_code': 400,
            'sub_category': 'login',
        },
        981102: {
            'message': 'Token expired',
            'http_code': 400,
            'sub_category': 'login',
        },
        981103: {
            'message': 'Token corrupted',
            'http_code': 400,
            'sub_category': 'login',
        },
        981104: {
            'message': 'Missing uid in deserialized data',
            'http_code': 400,
            'sub_category': 'login',
        },
        981105: {
            'message': 'Not supported user type',
            'http_code': 400,
            'sub_category': 'login',
        },
        981106: {
            'message': 'Missing token',
            'http_code': 400,
            'sub_category': 'login',
        },
        981107: {
            'message': 'Wrong token format',
            'http_code': 400,
            'sub_category': 'login',
        },
        981201: {
            'message': 'Retrieving session_info from wechat error',
            'http_code': 400,
            'sub_category': 'login.wechat',
        },
        981202: {
            'message': 'No openid found in session_info returned from wechat',
            'http_code': 404,
            'sub_category': 'login.wechat',
        },
        981203: {
            'message': 'Wechat user not found by id',
            'http_code': 404,
            'sub_category': 'login.wechat',
        },
        981251: {
            'message': 'Creating user in db error',
            'http_code': 500,
            'sub_category': 'login.wechat',
        },
        981301: {
            'message': 'Missing username while admin login',
            'http_code': 400,
            'sub_category': 'login.admin',
        },
        981302: {
            'message': 'Missing password while admin login',
            'http_code': 400,
            'sub_category': 'login.admin',
        },
        981303: {
            'message': 'Specified admin not exists',
            'http_code': 404,
            'sub_category': 'login.admin',
        },
        981304: {
            'message': 'Password not matched',
            'http_code': 400,
            'sub_category': 'login.admin',
        },
        981305: {
            'message': 'admin not found by id',
            'http_code': 404,
            'sub_category': 'login.admin',
        },
        981401: {
            'message': 'Permission denied',
            'http_code': 403,
            'sub_category': 'authorization',
        },
    }
